package dbclase;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.HashSet;

/**
 * HashSet que almacena objetos de la clase interna FirstPhoneme.
 * Se construye a partir de un fichero con formato de alineamiento fonético.
 * que representan los fonemas iniciales de palabra.
 * Los fonemas almacenados son los iniciales de toda palabra que tiene asignado un fonema con su duración.
 * Las palabras que no tienen fonema y duración son contadas como palabras no válidas
 *
 * @author gbg
 */
public class FirstPhoneneSet extends HashSet<FirstPhoneneSet.FirstPhoneme>{

    /** Clase que representa un fonema inicial de palabra por la palabra y su duración.
     */
    public static class FirstPhoneme{
        private final String palabra;
        private final double duracion;

        public FirstPhoneme(String palabra, double duracion) {
            this.palabra = palabra;
            this.duracion = duracion;
        }
    }

    // Contador de palabras inválidas
    private int palabrasInvalidas=0;

    /**
     * Constructor a partir de un fichero con formato de alineamiento fonético.
     * @param fileUrl la URL del fichero.
     * @throws IOException no es posible leer el fichero o el parámetro fileUrl no es una URL está bien formada.
     */
    public FirstPhoneneSet(String fileUrl) throws IOException {
        // Se genera un objeto URL con la string, se le pide un stream de datos y se envuelve convenientemente.
        BufferedReader br=new BufferedReader(new InputStreamReader(new URL(fileUrl).openStream()));
        // Leemos el fichero linea a linea
        String linea;
        while ( (linea=br.readLine()) !=null) {
            // Rechazamos todas las lineas que con coincidan con inicio de palabra.
            if (!linea.matches(".*\\|[^\\|]+\\|.*\\|.*")) continue;
            // Si no tienen valores de tiempo es palabra inválida
            if (linea.matches(".*\\|[^\\|]+\\|\\|.*")) {palabrasInvalidas++;continue;}
            // Obtenemos los tres valores que nos interesan en un array de String 
            String[] arr=linea.replaceFirst(".*\\|([^\\|]+)\\|(.+) (.+) .*\\|.*", "$1 $2 $3").split(" ");
            // y almacenamos un objeto FirstPhoneme con la palabra y la duración (diferencia de tiempos)
            add(new FirstPhoneme(arr[0],Double.parseDouble(arr[2])-Double.parseDouble(arr[1])));
           }
    }

    /**
     * Cálculo de la media de las duraciones de todos los elementos del conjunto.
     * @return el valor de la media.
     */
    public double media(){
        double sumatorio=0.0;
        for (FirstPhoneme p:this) sumatorio+=p.duracion;
        return sumatorio/size();
    }

    /**
     * Cáculo de la desviación típica de las duraciones de todos los elementos del conjunto.
     * @return el valor de la desviación típica.
     */
    public double desviacion(){
        double sumatorio=0.0, media=media();
        for (FirstPhoneme p:this) sumatorio+=(p.duracion-media)*(p.duracion-media);
        return Math.sqrt(sumatorio/(size()-1));
    }

    /**
     * Número de palabras en el conjunto (las palabras válidas)
     * @return el número de palabras válidas.
     */
    public int palabrasValidas(){ return size(); }

    /**
     * Número de palabras totales en el fichero procesado (suma de válidas e inválidas)
     * @return el número de palabras totales en el fichero procesado.
     */
    public int palabrasTotales(){ return palabrasValidas() + palabrasInvalidas; }
}
